# -*- coding: utf-8 -*-

import pygame 
import pygame.locals
import sys

C = 100
W = 3*C
DRAW = u'平'

class Game:

    def __init__( self, n, intelligence):
        self.bg_color = (255,255,255)
        self.line_color = (0,0,0)
        self.win_color = (200,200,0)
        self.cross = pygame.image.load( 'cross.png')
        self.circle = pygame.image.load( 'circle.png')
        self.cross = self.cross = pygame.transform.scale( self.cross, (C,C))
        self.circle = pygame.transform.scale( self.circle, (C,C))
        self.conf = dict( ((a,b), None) for a in range(n) for b in range(n))
        self.state = 'x'
        self.decide = intelligence
        self.n = n
        self.winning_line = []
        
    def main( self):
        
        # Here comes the event loop
        while True:
            # run this loop 25 times per second
            pygame.time.Clock().tick(25)
            # check for events
            for e in pygame.event.get():
                if e.type == pygame.locals.QUIT:
                    pygame.quit()
                # if e.type == pygame.KEYDOWN and e.key in KEYS:
                #     a,b = KEYS[e.key]

                if e.type == pygame.MOUSEBUTTONUP:
                    x,y = e.pos
                    a,b = x/C, y/C
                    self.mark( (a,b))

            self.pc_move()
            
            # update the frame
            screen.fill( self.bg_color)
            for a,b in self.winning_line:
                r = pygame.Rect( a*C, b*C, C, C)
                pygame.draw.rect( screen, self.win_color, r)
            for a,b in self.conf:
                if self.conf[(a,b)] == 'x':
                    screen.blit( self.cross, (a*C,b*C))
                if self.conf[(a,b)] == 'o':
                    screen.blit( self.circle, (a*C,b*C))
            for k in range(1, self.n):
                pygame.draw.line( screen, self.line_color, (k*C,0), (k*C,W), 3)
                pygame.draw.line( screen, self.line_color, (0,k*C), (W,k*C), 3)
            pygame.display.flip()


    def mark( self, c):
        a,b = c
        if self.state != 'x' or self.conf[(a,b)]:
            return
        self.conf[(a,b)] = 'x'
        if self.is_over():
            self.state = 'end'
        else:
            self.state = 'o'

    def pc_move(self):
        if self.state != 'o':
            return
        a,b = self.decide( self.conf)
        self.conf[(a,b)] = 'o'
        if self.is_over():
            self.state = 'end'
        else:
            self.state = 'x'

    def _lines( self):
        n = self.n
        h = [ [ (a,b) for b in range(n)] for a in range(n)]
        v = [ [ (a,b) for a in range(n)] for b in range(n)]
        d = [ [ (a,a) for a in range(n)], \
              [ (a,n-1-a) for a in range(n)]]
        return h + v + d
    
    def is_over( self):
        # check whether there is a winner
        counter = 2*self.n + 2 
        for l in self._lines():
            s = map( lambda c: self.conf[c], l)
            if 'x' in s and 'o' in s:
                counter -= 1
            elif s == ['x']*self.n or s == ['o']*self.n:
                self.winning_line = l
                return True
        # or a draw 
        if counter == 0:
            return True        
        return False

        
def _jerk( conf):
    return (0,0)

def _simple( conf):
    for k in conf:
        if conf[k] == None:
            return k

def _superhero( conf):
    # Exercise until spring 2016:
    # Implement the 'best move' strategy,
    # see https://en.wikipedia.org/wiki/Tic-tac-toe#Strategy
    pass


if __name__ == '__main__':

    try:
        n = int( sys.argv[1])
    except:
        n = 3

    W = C*n
    pygame.init()
    screen = pygame.display.set_mode( (W, W))
    Game( n, _simple).main()
    # Game( n, _superhero).main()
    pygame.quit()
